<?php
/**
 * Yii blog
 *
 * @author Marc Oliveras Galvez <admin@oligalma.com> 
 * @link http://www.oligalma.com
 * @copyright 2016 Oligalma
 * @license New BSD License
 */
 
class Category extends CActiveRecord
{
	const STATUS_UNPUBLISHED=1;
	const STATUS_PUBLISHED=2;
	
    public $maxId;
    
	/**
	 * Returns the static model of the specified AR class.
	 * @return CActiveRecord the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{category}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('id, name, status, language', 'required'),
			array('id, category_id', 'numerical', 'min' => 1 , 'max' => 999),
			array('id, name, status, category_id, language', 'safe', 'on'=>'search'),
			array('name','filter','filter'=>array($obj=new CHtmlPurifier(),'purify')),
	        array('id', 'unique', 'criteria'=>array(
	            'condition'=>'language=:language',
	            'params'=>array(
	                ':language'=>$this->language,	                
	            )
	        ), 'on' => 'update', 'message' => 'This id and language already exist.'),
	        array('id', 'checkCategory', 'on' => 'insert')
		);
	}
	
	public function checkCategory($attribute,$params)
	{
		$model = Category::model()->find('id = ? AND language = ?', array($this->id, $this->language));
		if($model != null)
			$this->addError('id','This id and language already exist.');	
	}

	
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'posts' => array(self::HAS_MANY, 'Post', array('category_id', 'language')),
			'categories' => array(self::HAS_MANY, 'Category', array('category_id', 'language'),'order' => 'LOWER(name) ASC'),
			'category' => array(self::BELONGS_TO, 'Category', array('category_id', 'language')),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Message::getTranslation(27),
			'name' => Message::getTranslation(10),
			'language' => Message::getTranslation(35),
			'category_id' => Message::getTranslation(40),
			'status' => Message::getTranslation(30),
		);
	}

	/**
	 * Retrieves the list of categories based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the needed posts.
	 */
	public function search()
	{
		$criteria=new CDbCriteria;
		$criteria->alias='c';
		$criteria->with = array('category'=>array('alias'=>'p'));
		$criteria->compare('c.id',$this->id);
		$criteria->compare('c.name',$this->name,true);
		$criteria->compare('p.name', $this->category_id,true);
		$criteria->compare('c.language',$this->language);
		$criteria->compare('c.status',$this->status);

		return new CActiveDataProvider('Category', array(
			'criteria'=>$criteria,
			'sort'=>array(
				'defaultOrder'=>'LOWER(c.name) ASC',
			),
			'pagination'=>array(
				'pageSize'=>Parameter::getValue(10),
			),
		));
	}
	
	public static function getCategoriesOptions($language)
	{
		$categories = Category::model()->findAll(array('condition' => 'language =\'' . $language . '\' AND category_id IS NULL','order' => 'LOWER(name) ASC'));
		return self::getListData($categories, 0);
	}

	private static function getListData($categories, $level)
	{
		$listData = array();
			
		foreach($categories as $category)
		{
			$lines = '';
			for($i = 0; $i < $level; $i++)
				$lines .= '--';
			$listData[$category->id] = $lines . $category->name;
			$listData += self::getListData($category->categories, $level + 1);
		}
		
		return $listData;
	}
	
	protected function beforeDelete()
	{
		if(count($this->posts) > 0 || count($this->categories) > 0)
			return false;
		
		return true;
	}

	public function getSubcategoriesIds()
	{
		$subcategoriesids = self::getSubcategoriesIdsR($this->categories);
		return $this->id . ($subcategoriesids === '' ? '' : $subcategoriesids);
	}
		
	public static function getSubcategoriesIdsR($categories)
	{
		$subcategories = '';
		foreach($categories as $category)
		{
			if($category->status == Category::STATUS_PUBLISHED)
			{
				$subcategories .= ',' . $category->id;
				$subcategories .= self::getSubcategoriesIdsR($category->categories);
			}
		}
		return $subcategories;
	}
	
	public function areParentCategoriesPublished()
	{
		$categories_published = true;
		$category = $this;
		while(true)
		{
			if($category->status == Category::STATUS_UNPUBLISHED)
			{
				$categories_published = false;
				break;
			}
			if(is_null($category->category_id))
				break;
			$category = $category->category;
		}
		return $categories_published;
	}		
}
	