<?php
/**
 * Yii blog
 *
 * @author Marc Oliveras Galvez <admin@oligalma.com> 
 * @link http://www.oligalma.com
 * @copyright 2016 Oligalma
 * @license New BSD License
 */
 
class Post extends CActiveRecord
{
	const STATUS_DRAFT=1;
	const STATUS_PUBLISHED=2;
	const STATUS_ARCHIVED=3;

	private $_oldTags;
	public $oldId;
	public $oldCategory;
	public $oldLanguage;
	public $postyear; // Used in the archives widget
	public $postmonth; // Used in the archives widget
	public $postspermonth; // Used in the archives widget
	public $maxId;
	
	/**
	 * Returns the static model of the specified AR class.
	 * @return CActiveRecord the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{post}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		$p = new CHtmlPurifier();
		//$p->options = array('HTML.AllowedCommentsRegexp'=> '/.*/');
		$p->options = array('HTML.AllowedComments'=> array('pagebreak'));	
		return array(
			array('id, title, content, status, language, category_id', 'required'),
			array('id', 'numerical', 'min' => 1, 'max' => 9999999999),
			array('status', 'in', 'range'=>array(1,2,3)),
			array('title', 'length', 'max'=>128),
			array('tags', 'match', 'pattern'=>'/^[\p{L}\s,0-9]+$/u', 'message'=>Message::getTranslation(101)),
			array('tags', 'normalizeTags'),
			array('meta_keywords, meta_description', 'default', 'setOnEmpty' => true, 'value' => null),
			array('id, title, status, category_id, language, user_id', 'safe', 'on'=>'search'),
			array('title, tags, meta_keywords, meta_description','filter','filter'=>array(new CHtmlPurifier(),'purify')),
			array('content','filter','filter'=>array($p,'purify')),
	        array('id', 'checkPost', 'on' => 'insert, update'),
		);
	}

	public function checkPost($attribute,$params)
	{
		if($this->id !== $this->oldId || $this->category_id !== $this->oldCategory || $this->language !== $this->oldLanguage)
		{
			$model = Post::model()->find('id = ? AND category_id = ? AND language = ?', array($this->id, $this->category_id, $this->language));
			if($model != null)
				$this->addError('id',Message::getTranslation(176));
		}
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'user' => array(self::BELONGS_TO, 'User', 'user_id'),
			'comments' => array(self::HAS_MANY, 'Comment', array('post_id', 'category_id', 'language'), 'condition'=>'comments.status='.Comment::STATUS_APPROVED . ' AND comments.comment_id IS NULL', 'order'=>'comments.create_time DESC'),
			'category' => array(self::BELONGS_TO, 'Category', array('category_id', 'language')),
			'lang' => array(self::BELONGS_TO, 'Language', array('language')),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Message::getTranslation(27),
			'title' => Message::getTranslation(28),
			'content' => Message::getTranslation(29),
			'tags' => Message::getTranslation(4),
			'status' => Message::getTranslation(30),
			'create_time' => Message::getTranslation(31),
			'update_time' => Message::getTranslation(32),
			'user_id' => Message::getTranslation(33),
			'category_id' => Message::getTranslation(34),
			'language' => Message::getTranslation(35),
			'meta_keywords' => Message::getTranslation(36),
			'meta_description' => Message::getTranslation(37),
			'commentCount' => '#' . substr(Message::getTranslation(17),0,3),
		);
	}

	public function getCommentCount()
	{
		return Comment::model()->count('post_id = ? AND language = ? AND status = ?', array($this->id, $this->language, Comment::STATUS_APPROVED));
	}
	
	/**
	 * @return string the URL that shows the detail of the post
	 */
	public function getUrl()
	{
		$url = array();
		$url['post'] = $this->id;
		$url['category'] = $this->category_id;
		$url['title'] = $this->sanitizeString($this->title);
		$url['lang'] = $this->language;
		return Yii::app()->createUrl('blog/post/view', $url);
	}

	public function getAbsoluteUrl()
	{
		$url = array();
		$url['post'] = $this->id;
		$url['category'] = $this->category_id;
		$url['title'] = $this->sanitizeString($this->title);
		$url['lang'] = $this->language;
		return Yii::app()->createAbsoluteUrl('blog/post/view', $url);
	}

	public function sanitizeString($string)
	{
	
	    $string = trim($string);
	
	    $string = str_replace(
	        array('á', 'à', 'ä', 'â', 'ª', 'Á', 'À', 'Â', 'Ä'),
	        array('a', 'a', 'a', 'a', 'a', 'A', 'A', 'A', 'A'),
	        $string
	    );
	
	    $string = str_replace(
	        array('é', 'è', 'ë', 'ê', 'É', 'È', 'Ê', 'Ë'),
	        array('e', 'e', 'e', 'e', 'E', 'E', 'E', 'E'),
	        $string
	    );
	
	    $string = str_replace(
	        array('í', 'ì', 'ï', 'î', 'Í', 'Ì', 'Ï', 'Î'),
	        array('i', 'i', 'i', 'i', 'I', 'I', 'I', 'I'),
	        $string
	    );
	
	    $string = str_replace(
	        array('ó', 'ò', 'ö', 'ô', 'Ó', 'Ò', 'Ö', 'Ô'),
	        array('o', 'o', 'o', 'o', 'O', 'O', 'O', 'O'),
	        $string
	    );
	
	    $string = str_replace(
	        array('ú', 'ù', 'ü', 'û', 'Ú', 'Ù', 'Û', 'Ü'),
	        array('u', 'u', 'u', 'u', 'U', 'U', 'U', 'U'),
	        $string
	    );
	
	    $string = str_replace(
	        array('ñ', 'Ñ', 'ç', 'Ç'),
	        array('n', 'N', 'c', 'C',),
	        $string
	    );
	
	    $string = str_replace(
	        array("\\", "¨", "º", "-", "~",
	             "#", "@", "|", "!", "\"",
	             "·", "$", "%", "&", "/",
	             "(", ")", "?", "'", "¡",
	             "¿", "[", "^", "`", "]",
	             "+", "}", "{", "¨", "´",
	             ">", "< ", ";", ",", ":",
	             "."),
	        '',
	        $string
	    );
		
		$string = strtolower($string);
		
		$string = str_replace(' ', '-', $string);
	
	    return $string;
	}

	/**
	 * @return array a list of links that point to the post list filtered by every tag of this post
	 */
	public function getTagLinks()
	{
		$links=array();
		foreach(Tag::string2array($this->tags) as $tag)
			$links[]=CHtml::link(CHtml::encode($tag), array('post/index', 'tag'=>preg_replace(array('|\s+|','/\W-/'), array('-','-'), $tag), 'lang' => $this->language), array('rel' => Parameter::getValue(32) ? 'nofollow' : null));
		return $links;
	}

	/**
	 * Normalizes the user-entered tags.
	 */
	public function normalizeTags($attribute,$params)
	{
		$this->tags=Tag::array2string(array_unique(Tag::string2array($this->tags)));
	}

	/**
	 * Adds a new comment to this post.
	 * This method will set status and post_id of the comment accordingly.
	 * @param Comment the comment to be added
	 * @return boolean whether the comment is saved successfully
	 */
	public function addComment($comment)
	{
		if(Yii::app()->user->checkAccess('admin'))
		{
			$comment->status=Comment::STATUS_APPROVED;
		}
		else
		{
			if(Parameter::getValue(6))
				$comment->status=Comment::STATUS_PENDING;
			else
				$comment->status=Comment::STATUS_APPROVED;
		}
		
		$comment->post_id=$this->id;
		$comment->category_id=$this->category_id;
		$comment->language=$this->language;
		$comment->user_id = Yii::app()->user->id;
		
		return $comment->save();
	}

	/**
	 * This is invoked when a record is populated with data from a find() call.
	 */
	protected function afterFind()
	{
		parent::afterFind();
		$this->_oldTags=$this->tags;
		$this->oldId = $this->id;
		$this->oldCategory = $this->category_id;
		$this->oldLanguage = $this->language;
	}

	/**
	 * This is invoked before the record is saved.
	 * @return boolean whether the record should be saved.
	 */
	protected function beforeSave()
	{
		if(parent::beforeSave())
		{			
			if($this->meta_keywords === "")
				$this->meta_keywords = null;
			if($this->meta_description === "")
				$this->meta_description = null;
			
			if($this->isNewRecord)
			{
				$this->create_time=$this->update_time=time();
				$this->user_id=Yii::app()->user->id;
			}
			else
				$this->update_time=time();
		
			// Send the post to subscribed users....................
			if(Parameter::getValue(30) && !$this->sent && $this->status == Post::STATUS_PUBLISHED)
			{
				if($this->category->areParentCategoriesPublished())
				{
					$subscriptions = Subscription::model()->findAll(
						array('condition' => 'category_id IN (' . $this->category->getSubcategoriesIds() . ') AND language = ?',
								'distinct' => true,
								'select' => 'user_id',
								'params' => array($this->language)));				
					foreach($subscriptions as $subscription)
					{
						$user = User::model()->find(array('condition' => 'id=:id AND subscribe=1 AND status=:status', 'params' => array(':id' => $subscription->user_id, ':status' => User::STATUS_ACTIVE)));
						if(!is_null($user))
						{
							$user->sendNews($this->title, $this->content);
						}
					}
				}
				$this->sent = 1;
			}
			// END Send the post to subscribed users.................				
			return true;
		}
		else
			return false;
	}

	/**
	 * This is invoked after the record is saved.
	 */
	protected function afterSave()
	{
		parent::afterSave();
		
		Tag::model()->updateFrequency($this->_oldTags, $this->tags, $this->language);
	}

	/**
	 * This is invoked after the record is deleted.
	 */
	protected function beforeDelete()
	{
		if(count($this->getAllComments()) == 0)
		{
			return true;
		}

		return false;
	}

	public function getIntroductoryText()
	{	
		$posbreak = strpos($this->content, '<!-- pagebreak -->');
		if($posbreak > 0)
			return substr($this->content,0,$posbreak) . ' [...]';
		else
		{		
			$retval = $this->content; // Just in case of a problem
			$array = explode(" ", $this->content);
			/* Already short enough, return the whole thing*/
			if (count($array) <= Parameter::getValue(5))
			{
				$retval = $this->content;
			}
			
			/* Need to chop of some words*/
			else
			{
				array_splice($array, Parameter::getValue(5));
				$retval = implode(" ", $array) . " [...]";
			}
			
			return strip_tags($retval);
		}
	}
	
	public function getAllComments()
	{
		return Comment::model()->findAll('post_id =? AND category_id = ? AND language = ?', array($this->id, $this->category_id, $this->language));
	}
	
	/**
	 * This is invoked after the record is deleted.
	 */
	protected function afterDelete()
	{
		parent::afterDelete();
		Tag::model()->updateFrequency($this->tags, '', $this->language);
	}

	/**
	 * Retrieves the list of posts based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the needed posts.
	 */
	public function search()
	{
		$criteria=new CDbCriteria;
		$criteria->alias='p';
		$criteria->with = array('category'=>array('alias'=>'c'), 'user'=>array('alias'=>'u'));
		$criteria->compare('p.id',$this->id);
		$criteria->compare('title',$this->title,true);
		$criteria->compare('c.name', $this->category_id,true);
		$criteria->compare('p.language',$this->language);
		$criteria->compare('p.status',$this->status);
		$criteria->compare('u.username',$this->user_id, true);
		if(Yii::app()->user->checkAccess('author'))
			$criteria->compare('p.user_id', Yii::app()->user->id);
		

		return new CActiveDataProvider('Post', array(
			'criteria'=>$criteria,
			'sort'=>array(
				'defaultOrder'=>'p.id DESC, p.language ASC',
			),
			'pagination'=>array(
				'pageSize'=>Parameter::getValue(8),
			),
		));
	}
	
	public function getStatusOptions()
	{
		if(Yii::app()->user->checkAccess('author') || Yii::app()->user->checkAccess('editor'))
			return CHtml::listData(Lookup::model()->findAll(array('condition' => 'code <> ' . Post::STATUS_PUBLISHED . ' AND type = "PostStatus" ','order' => 'code ASC')), 'code', 'name');
		else
			return CHtml::listData(Lookup::model()->findAll(array('condition' => 'type = "PostStatus"', 'order' => 'code ASC')), 'code', 'name');
	}
}