<?php
/**
 * Yii blog
 *
 * @author Marc Oliveras Galvez <admin@oligalma.com> 
 * @link http://www.oligalma.com
 * @copyright 2016 Oligalma
 * @license New BSD License
 */
 
class Tag extends CActiveRecord
{
	/**
	 * The followings are the available columns in table 'tbl_tag':
	 * @var integer $id
	 * @var string $name
	 * @var integer $frequency
	 */

	/**
	 * Returns the static model of the specified AR class.
	 * @return CActiveRecord the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{tag}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('name', 'required'),
			array('frequency', 'numerical', 'integerOnly'=>true),
			array('name', 'length', 'max'=>128),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'Id',
			'name' => 'Name',
			'frequency' => 'Frequency',
		);
	}

	/**
	 * Returns tag names and their corresponding weights.
	 * Only the tags with the top weights will be returned.
	 * @param integer the maximum number of tags that should be returned
	 * @return array weights indexed by tag names.
	 */
	public function findTagWeights($limit=20)
	{
		$categories = substr(Category::getSubcategoriesIdsR(Category::model()->findAll(array('condition' => 'language =:language AND category_id IS NULL','order' => 'LOWER(name) ASC', 'params' => array(':language' => Yii::app()->getLanguage())))),1);
		$tagsArray = array();
		$posts = Post::model()->findAll('language=? AND status=? AND category_id IN(' . $categories . ')', array(Yii::app()->getLanguage(), Post::STATUS_PUBLISHED));
		foreach($posts as $post)
		{
			$tagsArray2 = explode(',', $post->tags);
			foreach($tagsArray2 as $tag)
			{
				$tagsArray[] = '\'' . $tag . '\'';
			}
		}
		
		$models=$this->findAll(array(
			'condition' => 'language=\'' . Yii::app()->getLanguage() . '\' AND name IN('. implode(',', $tagsArray) . ')' ,
			'order'=>'frequency DESC',
			'limit'=>$limit,
		));

		$total=0;
		foreach($models as $model)
			$total+=$model->frequency;

		$tags=array();
		if($total>0)
		{
			foreach($models as $model)
				$tags[$model->name]=8+(int)(16*$model->frequency/($total+10));
			ksort($tags);
		}
		return $tags;
	}

	/**
	 * Suggests a list of existing tags matching the specified keyword.
	 * @param string the keyword to be matched
	 * @param integer maximum number of tags to be returned
	 * @return array list of matching tag names
	 */
	public function suggestTags($keyword,$limit,$language)
	{
		$tags=$this->findAll(array(
			'condition'=>'name LIKE :keyword AND language=\'' . $language . '\'',
			'order'=>'frequency DESC, Name',
			'limit'=>$limit,
			'params'=>array(
				':keyword'=>'%'.strtr($keyword,array('%'=>'\%', '_'=>'\_', '\\'=>'\\\\')).'%',
			),
		));
		$names=array();
		foreach($tags as $tag)
			$names[]=$tag->name;
		return $names;
	}

	public static function string2array($tags)
	{
		return preg_split('/\s*,\s*/',trim($tags),-1,PREG_SPLIT_NO_EMPTY);
	}

	public static function array2string($tags)
	{
		return implode(',',$tags);
	}

	public function updateFrequency($oldTags, $newTags, $language)
	{
		$oldTags=self::string2array($oldTags);
		$newTags=self::string2array($newTags);
		$this->addTags(array_values(array_diff($newTags,$oldTags)), $language);
		$this->removeTags(array_values(array_diff($oldTags,$newTags)), $language);
	}

	public function addTags($tags, $language)
	{
		$criteria=new CDbCriteria;
		$criteria->addInCondition('name',$tags);
		$criteria->addCondition('language=\'' . $language . '\'');
		$this->updateCounters(array('frequency'=>1),$criteria);
		foreach($tags as $name)
		{
			if(!$this->exists('name=:name AND language=:language',array(':name'=>$name, ':language'=>$language)))
			{
				$tag=new Tag;
				$tag->name=$name;
				$tag->language=$language;
				$tag->frequency=1;
				$tag->save();
			}
		}
	}

	public function removeTags($tags, $language)
	{
		if(empty($tags))
			return;
		$criteria=new CDbCriteria;
		$criteria->addInCondition('name',$tags);
		$criteria->addCondition('language=\'' . $language . '\'');
		$this->updateCounters(array('frequency'=>-1),$criteria);
		$this->deleteAll('frequency<=0 AND language=\'' . $language . '\'');
	}
}